#include <NTPClient.h>
#include <ESP8266WiFi.h>
#include <Ticker.h>
#include <WiFiUdp.h>
#include "IPGeolocation.h"
#include <ArduinoJson.h>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <RtcDS1307.h>
#include <Wire.h>
#include "OLEDDisplayUi.h"
#include "SH1106Wire.h", legacy include: `#include "SH1106.h"`
#include <Arduino.h>
#include <BMP180I2C.h>
#include "DHT22.h"
#include "RTCDS1307.h"
#include <SPI.h>
#include "ThingSpeak.h"
#include <Adafruit_Sensor.h>
#include <SoftwareSerial.h>

#define I2C_ADDRESS 0x77
#define countof(a) (sizeof(a) / sizeof(a[0]))

using namespace std;

// Declare bluethooth, rtc and wifi
SoftwareSerial btSerial(D4, D8);
RtcDS1307<TwoWire> Rtc(Wire);
WiFiClient  client;

// Define NTP Client to get time
WiFiUDP ntpUDP;
const long utcOffsetInSeconds = 0; // if i want to specify offset, but i don't need because i ger it from request
NTPClient timeClient(ntpUDP, "pool.ntp.org", utcOffsetInSeconds);

// For ThingsSpeak
unsigned long myChannelNumber = 2156648;
const char * myWriteAPIKey = "19HHR2VG0JQRTIXY";
String HOST = "api.thingspeak.com";
String PORT = "80";
String AP = "Ceas";
String PASS = "AP PASSWORD";

// WiFi credentials
// const char *ssid     = "Heroes";
// const char *password = "Nemrac1234";
const char *ssid     = "Idolul femeilor";
const char *password = "gase5300";

// Personal key for NTP
String Key = "919ec1f771724ca6aab4cfb55691b3f7";

// Timer variables for adding to ThingsSpeak
unsigned long lastTimeAdd = 0;
unsigned long timerAddDelay = 60000;

// Days of the week array
char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

// Global variable for time and date and time_offset and other flags for me
int ora;
int minut;
int secunda;
int timezone_offset;
int Luna;
int dayIndex;
int dayChanged;
int oraAlarma = -1;
int minutAlarma = -1;
int aSunat = 0;
long ts = 0;
int ok = 0;
long tsBuzzer = 0;

string dateString;
String day_of_the_week;
String day;
String month;
String year;
String city;
String country;
String time1;

bool period = 0;

// Initialize the OLED display using Wire library
SH1106Wire display(0x3c, SDA, SCL);
OLEDDisplayUi ui ( &display );

// Create an BMP180 object using the I2C interface
BMP180I2C bmp180(I2C_ADDRESS);

// Crete an dht22 humidity and temperature sensor
DHT22 dht22(D3); 

// Define display size and clock center and radius
int screenW = 128;
int screenH = 64;
int clockCenterX = screenW / 2;
int clockCenterY = ((screenH - 16) / 2) + 13;
int clockRadius = 23;
int displayOffset = 20;

int needsReconnect = 0;

// Clock overlay displaty is empty because it is mandatory and I dont need it
void clockOverlay(OLEDDisplay *display, OLEDDisplayUiState* state) {

}

// Analog clock display
void analogClockFrame(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) {

  // litle circle in the middle
  display->drawCircle(clockCenterX + x, clockCenterY + y, 2);

  //hour ticks
  for (int h = 0; h < 12; h++) {
      float angle = h * 30;
      angle = angle * PI / 180; // Convert degrees to radians
      int x1 = clockCenterX + x + (int)(sin(angle) * (clockRadius - 2));
      int y1 = clockCenterY + y - (int)(cos(angle) * (clockRadius - 2));
      int x2 = clockCenterX + x + (int)(sin(angle) * clockRadius);
      int y2 = clockCenterY + y - (int)(cos(angle) * clockRadius);
      
      if (h == 0)
        display->drawString(x1, y1 -2, "12");
      else if (h == 3)
        display->drawString(x1 + 2, y1, "3");
      else if (h == 6)
        display->drawString(x1, y1 + 2, "6");
      else if (h == 9)
        display->drawString(x1 - 2, y1, "9");
      else
        display->drawLine(x1, y1, x2, y2);
  }

  //display second hand
  float angle = secunda * 6 ;
  angle = angle * PI / 180; ; // Convert degrees to radians
  int x3 = ( clockCenterX + ( sin(angle) * ( clockRadius - 5 ) ) );
  int y3 = ( clockCenterY - ( cos(angle) * ( clockRadius - 5 ) ) );
  display->drawLine( clockCenterX + x , clockCenterY + y , x3 + x , y3 + y);

  // display minute hand
  angle = minut * 6 ;
  angle = angle * PI / 180 ; //Convert degrees to radians
  x3 = ( clockCenterX + ( sin(angle) * ( clockRadius - 8 ) ) );
  y3 = ( clockCenterY - ( cos(angle) * ( clockRadius - 8 ) ) );
  display->drawLine( clockCenterX + x , clockCenterY + y , x3 + x , y3 + y);

  // display hour hand
  angle = ora * 30 + (int)(minut/2); // minut / 2 bcz there is a difference of 30 between hours and minute is max 60
  angle = angle * PI / 180 ; // Convert degrees to radians
  x3 = ( clockCenterX + ( sin(angle) * ( clockRadius - 11 ) ) );
  y3 = ( clockCenterY - ( cos(angle) * ( clockRadius - 11 ) ) );
  display->drawLine( clockCenterX + x , clockCenterY + y , x3 + x , y3 + y);
}

// Digital clock and time + date 
void digitalClockFrame1(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) {
  String timenow = twoDigits(ora) + ":" + twoDigits(minut) + ":" + twoDigits(secunda);
  display->setTextAlignment(TEXT_ALIGN_CENTER);
  display->setFont(ArialMT_Plain_24);
  display->drawString(clockCenterX + x , clockCenterY + y, timenow );

  display->setTextAlignment(TEXT_ALIGN_CENTER);
  display->setFont(ArialMT_Plain_10);
  String today_date = day_of_the_week + " " + day + " " + month + " " + year;
  display->drawString(clockCenterX + x, clockCenterY + y - displayOffset, today_date);

  display->setTextAlignment(TEXT_ALIGN_CENTER);
  display->setFont(ArialMT_Plain_10);
  String location = city + " " + country;
  display->drawString(clockCenterX + x, clockCenterY + y - 1.5*displayOffset, location);
}

// Set frames for screen
FrameCallback frames[] = { analogClockFrame, digitalClockFrame1, displayTemperatureAndOthers};

// Frames number
int frameCount = 3;

// Overlays are statically drawn on top of a frame (nothing for me)
OverlayCallback overlays[] = { clockOverlay };
int overlaysCount = 1;

// Sensors measurements display
void displayTemperatureAndOthers(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) {
    // Start a temperature measurement
    float t = dht22.getTemperature();
    float h = dht22.getHumidity();

    display->setTextAlignment(TEXT_ALIGN_CENTER_BOTH);
    display->setFont(ArialMT_Plain_10);
  
    if (!bmp180.measureTemperature())
    {
      btSerial.println("Could not start temperature measurement");
      return;
    }

    // Wait for the measurement to finish. Proceed as soon as hasValue() returned true. 
    do
    {
    } while (!bmp180.hasValue());
    
    btSerial.print("Temperature: "); 
    btSerial.print(bmp180.getTemperature()); 
    btSerial.println(" degC");
    
    String temp = "";
    temp += "Temperature: " + String((bmp180.getTemperature() + dht22.getTemperature()) / 2) + "°C";
    display->drawString(clockCenterX + x, clockCenterY + y - displayOffset, temp);
    
    // start a pressure measurement. pressure measurements depend on temperature measurement, you should only start a pressure 
    // measurement immediately after a temperature measurement. 
    if (!bmp180.measurePressure())
    {
      btSerial.println("Could not start perssure measurement");
      return;
    }

    // wait for the measurement to finish. proceed as soon as hasValue() returned true. 
    do
    {
    } while (!bmp180.hasValue());

    btSerial.print("Pressure: "); 
    btSerial.print(bmp180.getPressure());
    btSerial.println("Pa");

    String airPressure = "";
    airPressure += "Air pressure: " + String(bmp180.getPressure()) + "Pa";
    display->drawString(clockCenterX + x, clockCenterY + y - displayOffset * 0.5, airPressure);

    btSerial.print("Pressure: "); 
    btSerial.print(bmp180.getPressure() * 9.86923267 / 1000000);
    btSerial.println("atm");

    String airPressureAtm = "";
    airPressureAtm += "Air pressure: " + String(bmp180.getPressure() * 9.86923267 / 1000000) + "atm";
    display->drawString(clockCenterX + x, clockCenterY + y, airPressureAtm);
    
    btSerial.print("Air humidity: "); 
    btSerial.print(dht22.getHumidity());
    btSerial.println("%");
  
    String humidity = "";
    humidity += "Air humidity: " + String(dht22.getHumidity()) + "%";
    display->drawString(clockCenterX + x, clockCenterY + y + displayOffset * 0.5, humidity);
}

// Utility function for clock display to print the leading 0s
String twoDigits(int digits) {
  if (digits < 10) {
    String i = '0' + String(digits);
    return i;
  }
  else {
    return String(digits);
  }
}

// Utility function to check if a string contains only digits and endline terminators
bool isStringDigitsOnly(const string& str) {
    for (char c : str) {
        if (!std::isdigit(c) && c != '\0' && c != '\n' && c != '\r') {
            return false;
        }
    }
    return true;
}

bool wasError(const char* errorTopic = "")
{
    uint8_t error = Rtc.LastError();
    if (error != 0)
    {
        btSerial.print("[");
        btSerial.print(errorTopic);
        btSerial.print("] WIRE communications error (");
        btSerial.print(error);
        btSerial.print(") : ");

        switch (error)
        {
        case Rtc_Wire_Error_None:
            btSerial.println("(none?!)");
            break;
        case Rtc_Wire_Error_TxBufferOverflow:
            btSerial.println("transmit buffer overflow");
            break;
        case Rtc_Wire_Error_NoAddressableDevice:
            btSerial.println("no device responded");
            break;
        case Rtc_Wire_Error_UnsupportedRequest:
            btSerial.println("device doesn't support request");
            break;
        case Rtc_Wire_Error_Unspecific:
            Serial.println("unspecified error");
            break;
        case Rtc_Wire_Error_CommunicationTimeout:
            btSerial.println("communications timed out");
            break;
        }
        return true;
    }
    return false;
}

int getLunaFromString(String m) {
  if (m == "Jan") {
    return 1;
  }
  if (m == "Feb") {
    return 2;
  }
  if (m == "Mar") {
    return 3;
  }
  if (m == "Apr") {
    return 4;
  }
  if (m == "May") {
    return 5;
  }
  if (m == "Jun") {
    return 6;
  }
  if (m == "Jul") {
    return 7;
  }
  if (m == "Aug") {
    return 8;
  }
  if (m == "Sep") {
    return 9;
  }
  if (m == "Oct") {
    return 10;
  }
  if (m == "Nov") {
    return 11;
  }
  if (m == "Dec") {
    return 12;
  }
}

void setup(){

  // Buzzer, button and led
  pinMode(D7, OUTPUT);
  pinMode(D6, INPUT_PULLUP);
  pinMode(D5, OUTPUT);

  // Set ui display
  ui.setTargetFPS(60);
  ui.setIndicatorPosition(TOP);
  ui.setIndicatorDirection(LEFT_RIGHT);
  ui.setFrameAnimation(SLIDE_LEFT);
  ui.setFrames(frames, frameCount);
  ui.setOverlays(overlays, overlaysCount);
  ui.setTimePerFrame(5000);
  ui.init();
  display.flipScreenVertically();

  Serial.begin(9600);
  btSerial.begin(9600);

  WiFi.begin(ssid, password);

  while ( WiFi.status() != WL_CONNECTED ) {
    delay ( 500 );
    Serial.print ( "." );
  }

  timeClient.begin();

  IPGeolocation location(Key);
  IPGeo IPG;
  location.updateStatus(&IPG);
  city = IPG.city;
  country = IPG.country;
  timezone_offset = IPG.offset;
  btSerial.println(IPG.city);
  btSerial.println(IPG.country);
  btSerial.println(IPG.country_code);
  btSerial.println(IPG.tz);
  btSerial.println(IPG.offset);
  btSerial.println(location.getResponse());

  string responseString = string(location.getResponse().c_str());
  
  // Find the position of "date_time_wti"
  size_t pos = responseString.find("\"date_time_wti\":\"");
  if (pos != std::string::npos) {
      // Extract the value between the double quotes
      size_t startPos = pos + 17;  // Length of "\"date_time_wti\":\""
      size_t endPos = responseString.find('\"', startPos);
      if (endPos != std::string::npos) {
          dateString = responseString.substr(startPos, endPos - startPos);
      }
  }
  btSerial.println(String(dateString.c_str()));

  istringstream iss(dateString);
  vector<string> tokens(istream_iterator<string>{iss},
                                  istream_iterator<string>());

  // Extract day, day, month, year, and hour
  string strDay1 = tokens[0];
  string strDay2 = tokens[1];
  string strMonth = tokens[2];
  string strYear = tokens[3];
  string strHour = tokens[4];

  // Extract the day
  day = String(strDay2.c_str());

  // Extract the month
  month = String(strMonth.c_str());

  // Extract the year
  year = String(strYear.c_str());

  // Extract the day of the week
  day_of_the_week = String(strDay1.substr(0, strDay1.length()-1).c_str());

  // Extract the time
  time1 = String(strHour.c_str());

  timeClient.update();

  // Index in the days_of_the_week array
  dayIndex = timeClient.getDay();
  day_of_the_week = daysOfTheWeek[timeClient.getDay()];
  ora = timeClient.getHours() + timezone_offset;
  minut = timeClient.getMinutes();
  secunda = timeClient.getSeconds();

  // I want to display 0 not 24
  if(ora == 24)
    ora = 0;

  // I sync rtc and web clock at 00:00 and I mark that the day of the week changed
  if(ora == 0)
    dayChanged = 1;

  btSerial.println(day);
  btSerial.println(month);
  btSerial.println(year);
  btSerial.println(day_of_the_week);
  btSerial.println(city);
  btSerial.println(country);
  btSerial.println(time1);
  btSerial.println(timezone_offset);
  btSerial.println(ora);
  btSerial.println(minut);
  btSerial.println(secunda);

  Luna = getLunaFromString(month);

  // Set the desired date and time
  Rtc.Begin();
  #if defined(WIRE_HAS_TIMEOUT)
    Wire.setWireTimeout(3000 /* us */, true /* reset_on_timeout */);
  #endif
  RtcDateTime compiled = RtcDateTime(__DATE__, __TIME__);
  printDateTime(compiled);
  Serial.println();
  if (!Rtc.IsDateTimeValid()) 
  {
      if (!wasError("setup IsDateTimeValid"))
      {
          // Common Causes:
          //    1) first time you ran and the device wasn't running yet
          //    2) the battery on the device is low or even missing

          Serial.println("RTC lost confidence in the DateTime!");
          // following line sets the RTC to the date & time this sketch was compiled
          // it will also reset the valid flag internally unless the Rtc device is
          // having an issue

          RtcDateTime dt(year.toInt(), Luna, day.toInt(), ora, minut, secunda);  // Example: May 14, 2023, 14:26:41
          Rtc.SetDateTime(dt);

      }
  }

  if (!Rtc.GetIsRunning())
  {
      if (!wasError("setup GetIsRunning"))
      {
          Serial.println("RTC was not actively running, starting now");
          Rtc.SetIsRunning(true);
      }
  }

  RtcDateTime now = Rtc.GetDateTime();
  if (!wasError("setup GetDateTime"))
  {
      if (now < compiled)
      {
          Serial.println("RTC is older than compile time, updating DateTime");
          RtcDateTime dt(year.toInt(), Luna, day.toInt(), ora, minut, secunda);  // Example: May 14, 2023, 14:26:41
          Rtc.SetDateTime(dt);
      }
      else if (now > compiled)
      {
          Serial.println("RTC is newer than compile time, this is expected");
      }
      else if (now == compiled)
      {
          Serial.println("RTC is the same as compile time, while not expected all is still fine");
      }
  }

  // never assume the Rtc was last configured by you, so
  // just clear them to your needed state
  Rtc.SetSquareWavePin(DS1307SquareWaveOut_Low); 
  wasError("setup SetSquareWavePin");
  RtcDateTime dt(year.toInt(), Luna, day.toInt(), ora, minut, secunda);  // Example: May 14, 2023, 14:26:41
  Rtc.SetDateTime(dt);

  // BMP180
  Wire.begin();

	//begin() initializes the interface, checks the sensor ID and reads the calibration parameters.  
	if (!bmp180.begin())
	{
		btSerial.println("begin() failed. check your BMP180 Interface and I2C Address.");
		while (1);
	}

	//reset sensor to default parameters.
	bmp180.resetToDefaults();

	//enable ultra high resolution mode for pressure measurements
	bmp180.setSamplingMode(BMP180MI::MODE_UHR);

  ThingSpeak.begin(client);  // Initialize ThingSpeak

}

void loop() {
  
  if (btSerial.available()) {
    // Read alarm time from phone
    char receivedData[256];
    int bytesRead = btSerial.readBytes(receivedData, sizeof(receivedData) - 1);
    receivedData[bytesRead] = '\0'; // Add null terminator to the end of the receivedData array
    string str(receivedData);
    size_t colonPos = str.find(':');
    
    // The correct format for alarm is HH:MM
    // Here I check for : in the input from phone 
    if(colonPos != std::string::npos) {
      // I parse what is before and after :
      string maybeHour = str.substr(0, colonPos);
      string maybeMinute = str.substr(colonPos + 1);

      btSerial.println(String(maybeHour.c_str()));
      btSerial.println(String(maybeMinute.c_str()));

      // I check those strings to contain only digits
      if (isStringDigitsOnly(maybeHour) && isStringDigitsOnly(maybeMinute)) {
        // Parse the substrings to integers
        int beforeValue = 0;
        int afterValue = 0;

        std::istringstream beforeStream(maybeHour);
        std::istringstream afterStream(maybeMinute);

        // I check if the strings could be converted to ints
        if (!(beforeStream >> beforeValue)) {
            btSerial.println("Alarm couldn't be set. Try this format HH:MM");
        }
        else if (!(afterStream >> afterValue)) {
            btSerial.println("Alarm couldn't be set. Try this format HH:MM");
        }
        else {
          // If the alarm was introduced correct I check that it is a valid time
          if(beforeValue >= 0 && beforeValue < 24 && afterValue >= 0 && afterValue < 60) {
            // If the alarm is valid I set it
            oraAlarma = beforeValue;
            minutAlarma = afterValue;
            String mesaj = "Alarm set at " + String(str.c_str());
            btSerial.println(mesaj);
          } else {
            btSerial.println("Alarm couldn't be set. Try a valid time");
          }
        }
      } else {
        btSerial.println("Alarm couldn't be set. Try this format HH:MM");
      } 
    } else {
      btSerial.println("Alarm couldn't be set. Try this format HH:MM");
    }
  }

  // If I press the button I want to change the frame displayed but only once if I keep the button pressed 
  if(digitalRead(D6) == LOW) {
    if(ok == 0)
      ui.nextFrame();
    ok = 1;
  }

  // When I release the button I reset the flag
  if(digitalRead(D6) == HIGH) {
    ok = 0;
  }

  // Get current date and time from DS1307 module each second
  if ((millis() - ts) >= 1000) {
    ts = millis();
    
    RtcDateTime now = Rtc.GetDateTime();
    ora = now.Hour();

    if(ora == 24)
      ora = 0;

    minut = now.Minute();
    secunda = now.Second();
    year = String(now.Year());
    Luna = now.Month();
    day = String(now.Day());

    // If I need to update the date 
    // If I have conection, I sync
    if(ora == 0 && dayChanged == 0 || needsReconnect == 1) {
      if(timeClient.update()) {
        
        dayIndex = timeClient.getDay();
        ora = (timeClient.getHours() + timezone_offset) % 24;
        minut = timeClient.getMinutes();
        secunda = timeClient.getSeconds();
        
        IPGeolocation location(Key);
        IPGeo IPG;
        location.updateStatus(&IPG);
        
        // I want to display 0 not 24
        if(ora == 24)
          ora = 0;

        string responseString = string(location.getResponse().c_str());
        
        // Find the position of "date_time_wti"
        size_t pos = responseString.find("\"date_time_wti\":\"");
        if (pos != std::string::npos) {
            // Extract the value between the double quotes
            size_t startPos = pos + 17;  // Length of "\"date_time_wti\":\""
            size_t endPos = responseString.find('\"', startPos);
            if (endPos != std::string::npos) {
                dateString = responseString.substr(startPos, endPos - startPos);
            }
        }
        btSerial.println(String(dateString.c_str()));

        istringstream iss(dateString);
        vector<string> tokens(istream_iterator<string>{iss},
                                        istream_iterator<string>());

        // Extract day, day, month, year, and hour
        string strDay1 = tokens[0];
        string strDay2 = tokens[1];
        string strMonth = tokens[2];
        string strYear = tokens[3];
        string strHour = tokens[4];

        // Extract the day
        day = String(strDay2.c_str());

        // Extract the month
        month = String(strMonth.c_str());

        // Extract the year
        year = String(strYear.c_str());

        Luna = getLunaFromString(month);
        
        RtcDateTime dt(year.toInt(), Luna, day.toInt(), ora, minut, secunda);
        Rtc.SetDateTime(dt);

      } else {
        dayIndex++;
        dayIndex %= 7;
      }
      if(needsReconnect == 1)
        needsReconnect = 0;
      else dayChanged = 1;
      day_of_the_week = daysOfTheWeek[dayIndex];
      
    }

    // Reset the flag for dayChanged
    if(ora == 1 && dayChanged == 1) {
      dayChanged = 0;
    }
    
    if (!Rtc.IsDateTimeValid()) 
      {
          if (!wasError("loop IsDateTimeValid")) {
              btSerial.println("RTC lost confidence in the DateTime!");
          }
      }

      now = Rtc.GetDateTime();
      if (!wasError("loop GetDateTime"))
      {
          printDateTime(now);
          btSerial.println();
      }

      // If it's time for my alarm
      if(ora == oraAlarma && minut == minutAlarma && secunda < 6) {
        
        aSunat = 1;
        tone(D7, 1000);

        // The buzzer will ring 3 times
        if ((millis() - tsBuzzer) >= 1500) {
          tsBuzzer = millis();
          noTone(D7);
        }

        digitalWrite(D5, HIGH);

      } else if (aSunat == 1) {

        digitalWrite(D5, LOW);
        aSunat = 0;
        noTone(D7);
        oraAlarma = -1;
        minutAlarma = -1;

      }

      ui.update();
  }

  // Add data from sensors to ThingsSpeak website
  if ((millis() - lastTimeAdd) > timerAddDelay) {
  
    // Connect or reconnect to WiFi
    if(WiFi.status() != WL_CONNECTED){
      btSerial.print("Attempting to connect");
      if(WiFi.status() != WL_CONNECTED){
        WiFi.begin(ssid, password);      
      }
      if(WiFi.status() != WL_CONNECTED) btSerial.println("\nCould not reconnect.");
      else btSerial.println("\nConnected.");
      needsReconnect = 1;
    }

    // Get a new temperature reading
    float temperatureBMP = bmp180.getTemperature();
    float pressureBMPPa = bmp180.getPressure();
    float pressureBMPatm = bmp180.getPressure() * 9.86923267 / 1000000;
    float temperatureDHT22 = dht22.getTemperature();
    float humidityDHT22 = dht22.getHumidity();
    
    // Set the fields with the values
    ThingSpeak.setField(1, (temperatureBMP + temperatureDHT22) / 2);
    ThingSpeak.setField(2, humidityDHT22);
    ThingSpeak.setField(3, pressureBMPPa);
    ThingSpeak.setField(4, pressureBMPatm);

    // Write to ThingSpeak
    int x = ThingSpeak.writeFields(myChannelNumber, myWriteAPIKey);

    if(x == 200){
      btSerial.println("Channel update successful.");
    }
    else{
      btSerial.println("Problem updating channel. HTTP error code " + String(x));
    }

    lastTimeAdd = millis();
  }

}

void printDateTime(const RtcDateTime& dt)
{
    char datestring[20];

    snprintf_P(datestring, 
            countof(datestring),
            PSTR("%02u/%02u/%04u %02u:%02u:%02u"),
            dt.Month(),
            dt.Day(),
            dt.Year(),
            dt.Hour(),
            dt.Minute(),
            dt.Second() );
    btSerial.println(datestring);
}